@@ -17,6 +17,14 @@
 #include <winsock2.h>
 #include <stdint.h>
 #else
+
+#if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__))
+#include <sys/param.h>
+#if defined(BSD)
+#include <netinet/in.h>
+#endif
+#endif
+
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/ip.h>
@@ -24,9 +32,11 @@
 #endif
 
 #include "../quisk.h"
+#include "../filter.h"
 
 #define GRAPH_DATA_SCALE	163
 #define MAX_UDP_INT16_T		600
+#define UDP_SEND_INT16		200
 
 #define REMOTE_DEBUG 0  //BMC TODO:  Make this a configuration option
 
@@ -43,320 +53,117 @@ static int packets_sent;
 static int packets_recd;
 
 // Receive stereo 16-bit pcm radio speaker sound on the control head via UDP
-int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamples, int nSamples)
+int read_remote_radio_sound_socket(complex double * cSamples)
 {
-	int i;
-	int bytes = 0;
+	int i, bytes, nInt16, nSamples;
+	int16_t buf[UDP_SEND_INT16];
 	double samp_r, samp_l;
-	int16_t * ptInt16;
-	int lapcount = 0;
 	struct timeval tm_wait;
 	fd_set fds;
-	SOCKET * sock = &control_head_sound_socket;
-	// static okay if only one instance runs, which should be good assumption when running remote
-	static char buf[128000];	// TODO:  Calculate based on remote sound latency setting
-	static int prebuf_samples = 48000 / 10; // 100 msec (TODO:  replace with calculation)
-	static int prebuf_done = 0;
-	static int rd_idx = 0;	// bytes
-	static int wr_idx = 0;	// bytes
-	static int wrap_idx;	// bytes
-	static int samps_in_buf = 0;	// samples (4 bytes per sample)
-	static int underrun_count = 0;	// samples
-	static size_t prior_pkt_size_max = 0;
+	static struct quisk_cHB45Filter HalfBand;
+	static struct quisk_cFilter cFiltInterp3;
+	static int init_filters=1;
 
-#if REMOTE_DEBUG > 0 //BMC debug -- summary every 200 calls
-	// measure/monitor tools:
-	static float callcount = 0;
-	static float sampcount_in = 0;
-	static float sampcount_out = 0;
-	static float nSamps = 0;
-	static uint64_t bunchcount = 0;
-	static float callcount_total = 0;
-	static float sampcount_in_total = 0;
-	static float sampcount_out_total = 0;
-	static float nSamps_total = 0;
-	static double delta_total = 0;
-	static double prior_ts = 0;
-#endif
-
-	if (*sock == INVALID_SOCKET)
+	if (control_head_sound_socket == INVALID_SOCKET)
 		return 0;
-
-#if REMOTE_DEBUG > 0 //BMC debug -- summary every 200 calls
-	nSamps += nSamples;
-	callcount++;
-#endif
+	if (init_filters) {
+		init_filters = 0;
+		memset(&HalfBand, 0, sizeof(struct quisk_cHB45Filter));
+		quisk_filt_cInit(&cFiltInterp3, quiskAudio24p3Coefs, sizeof(quiskAudio24p3Coefs)/sizeof(double));
+	}
 	// Signal far end (server) that we're ready (this sends our address/port to far end)
 	if (!control_head_sound_socket_started) {
-		QuiskPrintf("read_remote_radio_sound_socket() sending 'rr', *sock = %u\n", (u_int)*sock);
-		bytes = send(*sock, "rr\n", 3, 0);
+		QuiskPrintf("read_remote_radio_sound_socket() sending 'rr'\n");
+		bytes = send(control_head_sound_socket, "rr\n", 3, 0);
 		if (bytes != 3)
 			QuiskPrintf("read_remote_radio_sound_socket(), sendto(): %s\n", strerror(errno));
 	}
 
-	// read all available packets, one per loop, place into jitter-absorbing circular buffer
+	// read all available packets, one per loop
+	nSamples = 0;
 	while (1) {
-		int buftop;
-		char * wr_base;
-		size_t wr_len;
-		lapcount++;
-		// set up for direct receive into circular buffer
-		wr_base = &buf[wr_idx];
-		if (wr_idx > rd_idx || samps_in_buf == 0)
-			buftop = sizeof(buf);
-		else
-			buftop = rd_idx;
-		wr_len = buftop - wr_idx;	// contiguous (non-wrapping) space available in buffer
-		if (wr_len < prior_pkt_size_max && prior_pkt_size_max > 0) {
-			if (wr_idx > rd_idx || samps_in_buf == 0) {
-				// wrap before recv() so data expected in recv() fits within contiguous (non-wrapping) space
-				wrap_idx = wr_idx;
-				wr_idx = 0;
-				wr_base = &buf[0];
-				if (samps_in_buf == 0)
-					wr_len = sizeof(buf);
-				else
-					wr_len = rd_idx;
-#if REMOTE_DEBUG > 2 //BMC debug
-				QuiskPrintf("wrapping write at wrap_idx = %i, rd_idx = %i, wr_len = %li\n", wrap_idx, rd_idx, wr_len);
-#endif
-			}
-#if REMOTE_DEBUG > 0
-			else {
-				double now = QuiskTimeSec();
-				QuiskPrintf("%f, jitbuf overflow, reduce recv fm %u to %u, wr_idx %u, rd_idx %u, samps_in_buf %u, remainder 0s\n",
-					now, (u_int)prior_pkt_size_max, (u_int)wr_len, wr_idx, rd_idx, samps_in_buf);
-			}
-#endif
-		}
-
 		tm_wait.tv_sec = 0;
 		tm_wait.tv_usec = 0;
 		FD_ZERO (&fds);
-		FD_SET (*sock, &fds);
-		if (select(*sock + 1, &fds, NULL, NULL, &tm_wait) != 1) {
+		FD_SET (control_head_sound_socket, &fds);
+		if (select(control_head_sound_socket + 1, &fds, NULL, NULL, &tm_wait) != 1) {
 			//BMC QuiskPrintf("read_remote_radio_sound_socket(): select returned %i\n", retval);
-			bytes = 0;
 			break;
 		}
-		bytes = recv(*sock, wr_base, wr_len, 0);
+		bytes = recv(control_head_sound_socket, (char *)buf, UDP_SEND_INT16 * 2, 0);
 		if (bytes < 0) {
-			int err = errno;
-			if (err != EAGAIN && err != EWOULDBLOCK)
+			if (errno != EAGAIN && errno != EWOULDBLOCK)
 				QuiskPrintf("read_remote_radio_sound_socket(), recv(): %s\n", strerror(errno));
 			break;
 		}
 		if (bytes > 0) {
 			control_head_sound_socket_started = 1;
-			packets_recd++;
-			wr_idx += bytes;
-			if ((size_t)bytes > prior_pkt_size_max)
-				prior_pkt_size_max = bytes;
-			samps_in_buf += bytes / 4;	//TODO:  Calculate using sizeof
-#if REMOTE_DEBUG > 0 //BMC debug -- for summary every 200 calls
-			sampcount_in += bytes / 4;
-#endif
+			nInt16 = bytes / 2;
+			for (i = 0; i < nInt16; i += 2) {
+				samp_r = buf[i];
+				samp_l = buf[i + 1];
+				cSamples[nSamples++] = (samp_r + I * samp_l) / CLIP16 * CLIP32;
+			}
 		}
 	} // while(1)
-	if (!control_head_sound_socket_started  && bytes > 0) {
-		QuiskPrintf("read_remote_radio_sound_socket(): Started! samps_in_buf = %i\n", bytes / 4);
-		control_head_sound_socket_started = 1;
-	}
-	if (!prebuf_done) {
-		if (samps_in_buf >= prebuf_samples) {
-			prebuf_done = 1;
-			QuiskPrintf("read_remote_radio_sound_socket: Prebuf done, samps_in_buf = %i\n", samps_in_buf);
-		}
-	}
 
-	if (quisk_play_state > RECEIVE && dev->dev_index == t_Playback) {
-		// TX time for Radio Sound client ...
-		if (samps_in_buf > prebuf_samples * 2 && underrun_count > prebuf_samples / 4) {
-			// Network delays may cause radio sound jitter buffer underruns, but the incoming sound samples eventually
-			// arrive from the network, causing the jitter buffer to contain *more* samples than desired,
-			// perhaps even causing jitbuf overflows (so don't use underrun_count to calculate # samples to remove).
-			// Use Tx time to remove excess samples to keep playback latency short;
-			//     set rd_idx so there is prebuf_samples number of samples in buffer.
-#if REMOTE_DEBUG > 0
-			double now = QuiskTimeSec();
-			QuiskPrintf("%f, Removing excess jitbuf samples: underrun_count %i, samps_in_buf %i, rd_idx %i, wrap_idx %i, wr_idx %i\n",
-					now, underrun_count, samps_in_buf, rd_idx, wrap_idx, wr_idx);
-#endif
-			rd_idx = wr_idx - (prebuf_samples * 4);	// rd_idx = bytes; prebuf_samples = samples; 4 bytes per sample
-			if (rd_idx < 0)
-				rd_idx += wrap_idx;
-			samps_in_buf = prebuf_samples;
-			underrun_count = 0;
-#if REMOTE_DEBUG > 0
-			QuiskPrintf("%f, Removed excess jitbuf samples: samps_in_buf %i, rd_idx %i, wrap_idx %i,\n",
-					now, samps_in_buf, rd_idx, wrap_idx);
-#endif
-		}
-		if (quisk_active_sidetone == 4) {
-			// Overwrite remote sound with sidetone, starting at jitter buffer read position
-			int tmp_idx = rd_idx;
-			ptInt16 = (int16_t *)&buf[tmp_idx];
-			for (i = 0; i < nSamples; i++) {
-				int16_t * ptSamples = quisk_make_sidetone(dev, 0);
-				*ptInt16++ = *ptSamples;
-				*ptInt16++ = *ptSamples;
-				tmp_idx += 4;
-				if (tmp_idx >= wrap_idx && wrap_idx != 0) {
-					tmp_idx = 0;
-					ptInt16 = (int16_t *)&buf[tmp_idx];
-#if 0 //BMC debug
-					QuiskPrintf("wrapping sidetone at wrap_idx = %i\n", wrap_idx);
-#endif
-				}
-			}
-		}
-	}
+	nSamples = quisk_cInterpolate(cSamples, nSamples, &cFiltInterp3, 3);
+	nSamples = quisk_cInterp2HB45(cSamples, nSamples, &HalfBand);
 
-
-	if (prebuf_done) {
-		int numout = nSamples;
-		int newSamples = 0;
-		if (samps_in_buf < nSamples) {
-			double now = QuiskTimeSec();
-			numout = samps_in_buf;
-			underrun_count += (nSamples - samps_in_buf);
-			QuiskPrintf("%f, jitbuf low, reduce outsamps fm %i to %i, wr_idx %i, rd_idx %i, samps_in_buf %i, remainder 0s\n",
-				now, nSamples, numout, wr_idx, rd_idx, samps_in_buf);
-		}
-		ptInt16 = (int16_t *)&buf[rd_idx];
-		for (i = 0; i < numout; i++) {
-			samp_r = *ptInt16++;
-			samp_l = *ptInt16++;
-			cSamples[newSamples++] = (samp_r + I * samp_l) / CLIP16 * CLIP32;
-#if REMOTE_DEBUG > 0 //BMC debug -- for summary every 200 calls
-			sampcount_out++;
-#endif
-			samps_in_buf--;
-			rd_idx += 4;
-			if (rd_idx >= wrap_idx && wrap_idx != 0) {
-				rd_idx = 0;
-				ptInt16 = (int16_t *)&buf[rd_idx];
-#if REMOTE_DEBUG > 2 //BMC debug
-				QuiskPrintf("wrapping read at wrap_idx = %i\n", wrap_idx);
-#endif
-			}
-		}
-	}
-
-#if REMOTE_DEBUG > 1 //BMC debug -- per-call (perhaps multi-packet) statistics
-	if (lapcount != 1) {	// 1: 0 packets, 2: 1 packet, 3: 2 packets, etc.
-		double ts = QuiskTimeSec();
-		QuiskPrintf("%f: callcount = %lu, lapcount = %i, bytes = %i, nSamples = %i\n", ts, callcount, lapcount, bytes, nSamples);
-	}
-#endif
-
-#if REMOTE_DEBUG > 0 //BMC debug -- summary every 200 calls (a little over one second)
-	if (callcount >= 200) {
-		double new_ts = QuiskTimeSec();
-		double delta = new_ts - prior_ts;
-		prior_ts = new_ts;
-#if REMOTE_DEBUG > 2 //BMC raw numbers
-		QuiskPrintf("%f: read_remote_radio_sound_socket CURRENT calls: %lu, samps_in %lu, samps_out %lu, nSamps %lu, samps_in_buf %i, deltasec %f\n",
-			new_ts, callcount, sampcount_in, sampcount_out, nSamps, samps_in_buf, delta);
-#endif
-#if REMOTE_DEBUG > 1 //BMC raw numbers
-		QuiskPrintf("%f: read_remote_radio_sound_socket CURRENT RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i\n",
-			new_ts, callcount / delta, sampcount_in / delta, sampcount_out / delta, nSamps / delta, samps_in_buf);
-#endif
-		if (bunchcount > 0) {	// skip the initial bunch; prebuf may distort some numbers(?)
-			callcount_total += callcount;
-			sampcount_in_total += sampcount_in;
-			sampcount_out_total += sampcount_out;
-			nSamps_total += nSamps;
-			delta_total += delta;
-		}
-		if (bunchcount % 10 == 0 && bunchcount > 0) {
-#if REMOTE_DEBUG > 1 //BMC
-			QuiskPrintf("%f: read_remote_radio_sound_socket SUMMARY calls: %f, samps_in %f, samps_out %f, nSamps %f, samps_in_buf %i, deltasec %f\n",
-				new_ts, callcount_total, sampcount_in_total, sampcount_out_total, nSamps_total, samps_in_buf, delta_total);
-			QuiskPrintf("%f: read_remote_radio_sound_socket SUMMARY RATES (HZ): calls %f, samps_in %f, samps_out %f, nSamps %f\n",
-				new_ts, callcount_total / delta_total, sampcount_in_total / delta_total,
-				sampcount_out_total / delta_total, nSamps_total / delta_total);
-#else
-			float latency = (float)samps_in_buf / 48000.0;
-			QuiskPrintf("%f: read_remote_radio_sound_socket samps_in_buf %i, latency %f sec\n", new_ts, samps_in_buf, latency);
-#endif
-		}
-		bunchcount++;
-		callcount = 0;
-		sampcount_in = 0;
-		sampcount_out = 0;
-		nSamps = 0;
-	}
-#endif
-	// Always return same # samples sent in, even if we didn't change any
 	return nSamples;
 }
 
-
-
 // Receive stereo 16-bit pcm microphone samples at the remote radio via UDP
-int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSamples)
+int read_remote_mic_sound_socket(complex double * cSamples)
 {
-	int i;
-	int bytes = 0;
+	int i, bytes, nInt16, nSamples;
+	int16_t buf[UDP_SEND_INT16];
 	double samp_r, samp_l;
-	int16_t * ptInt16;
-	int rd_idx = 0;	// bytes
-	int wr_idx = 0;	// bytes
-	int samps_in_buf = 0;	// samples (4 bytes per sample)
-	int lapcount = 0;
 	struct timeval tm_wait;
 	fd_set fds;
-	SOCKET * sock = &remote_radio_sound_socket;
-	// static okay if only one instance runs, which should be good assumption when running remote
-	static char buf[128000];	// TODO:  Calculate based on ???
+	static struct quisk_cHB45Filter HalfBand;
+	static struct quisk_cFilter cFiltInterp3;
+	static int init_filters=1;
 
-	if (*sock == INVALID_SOCKET)
+	if (remote_radio_sound_socket == INVALID_SOCKET)
 		return 0;
+	if (init_filters) {
+		init_filters = 0;
+		memset(&HalfBand, 0, sizeof(struct quisk_cHB45Filter));
+		quisk_filt_cInit(&cFiltInterp3, quiskAudio24p3Coefs, sizeof(quiskAudio24p3Coefs)/sizeof(double));
+	}
 
-	// read all available packets, one per loop, place into jitter-absorbing circular buffer
+	// read all available packets, one per loop
+	nSamples = 0;
 	while (1) {
-		int buftop = sizeof(buf);
-		char * wr_base;
-		size_t wr_len;
-		lapcount++;
-		// set up for direct receive into linear buffer
-		wr_base = &buf[wr_idx];
-		wr_len = buftop - wr_idx;	// remaining contiguous space available in buffer
-
 		tm_wait.tv_sec = 0;
 		tm_wait.tv_usec = 0;
 		FD_ZERO (&fds);
-		FD_SET (*sock, &fds);
-		if (select(*sock + 1, &fds, NULL, NULL, &tm_wait) != 1) {
+		FD_SET (remote_radio_sound_socket, &fds);
+		if (select(remote_radio_sound_socket + 1, &fds, NULL, NULL, &tm_wait) != 1) {
 			//BMC QuiskPrintf("read_remote_mic_sound_socket(): select returned %i\n", retval);
-			bytes = 0;
 			break;
 		}
-		bytes = recv(*sock, wr_base, wr_len, 0);
+		bytes = recv(remote_radio_sound_socket, (char *)buf, UDP_SEND_INT16 * 2, 0);
 		if (bytes < 0) {
-			int err = errno;
-			if (err != EAGAIN && err != EWOULDBLOCK)
+			if (errno != EAGAIN && errno != EWOULDBLOCK)
 				QuiskPrintf("read_remote_mic_sound_socket(), recv(): %s\n", strerror(errno));
 			break;
 		}
 		if (bytes > 0) {
-			packets_recd++;
-			wr_idx += bytes;
-			samps_in_buf += bytes / 4;	//TODO:  Calculate using sizeof
+			nInt16 = bytes / 2;
+			for (i = 0; i < nInt16; i += 2) {
+				samp_r = buf[i];
+				samp_l = buf[i + 1];
+				cSamples[nSamples++] = (samp_r + I * samp_l) / CLIP16 * CLIP32;
+			}
 		}
 	} // while(1)
 
-	ptInt16 = (int16_t *)&buf[rd_idx];
-	for (i = 0; i < samps_in_buf; i++) {
-		samp_r = *ptInt16++;
-		samp_l = *ptInt16++;
-		cSamples[i] = (samp_r + I * samp_l) / CLIP16 * CLIP32;
-		rd_idx += 4;
-	}
+	nSamples = quisk_cInterpolate(cSamples, nSamples, &cFiltInterp3, 3);
+	nSamples = quisk_cInterp2HB45(cSamples, nSamples, &HalfBand);
 
-	return samps_in_buf;
+	return nSamples;
 }
 // Send stereo 16-bit pcm sound via UDP
 // This code acts as UDP server for radio sound (on remote radio) or mic sound (on control head)
@@ -366,22 +173,39 @@ int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSample
 // Send microphone samples from the control head to the remote radio
 void send_remote_mic_sound_socket(complex double * cSamples, int nSamples)
 {
-	int i, buffer_index, last;
-	int16_t buffer[MAX_UDP_INT16_T];
+	int i;
 	ssize_t sent;
+	static int buffer_index=0;
+	static int16_t buffer[UDP_SEND_INT16];
+	static struct quisk_cHB45Filter HalfBand;
+	static struct quisk_cFilter cFiltDecim3;
+	static int init_filters=1, size_cBuf=0;
+	static complex double * cBuf=NULL;
 
+	if (init_filters) {
+		init_filters = 0;
+		memset(&HalfBand, 0, sizeof(struct quisk_cHB45Filter));
+		quisk_filt_cInit(&cFiltDecim3, quiskAudio24p3Coefs, sizeof(quiskAudio24p3Coefs)/sizeof(double));
+	}
+	if (nSamples > size_cBuf) {
+		size_cBuf = nSamples;
+		cBuf = (complex double *)realloc(cBuf, size_cBuf * sizeof(complex double));
+	}
 	if (control_head_sound_socket == INVALID_SOCKET)
 		return;
 
 	if (!control_head_sound_socket_started)
 		return;
-	// Convert format from complex double to stereo pairs of 16-bit PCM samples, send to remote radio
-	buffer_index = 0;
-	last = nSamples - 1;
-	for (i = 0; i <= last; i++) {
-		buffer[buffer_index++] = (int16_t)(creal(cSamples[i]) * (double)CLIP16 / CLIP32);
-		buffer[buffer_index++] = (int16_t)(cimag(cSamples[i]) * (double)CLIP16 / CLIP32);
-		if (buffer_index >= MAX_UDP_INT16_T || i == last) {
+	memcpy(cBuf, cSamples, nSamples * sizeof(complex double));	// Do not alter cSamples
+	// Reduce sample rate from 48 to 8 ksps
+	nSamples = quisk_cDecim2HB45(cBuf, nSamples, &HalfBand);
+	nSamples = quisk_cDecimate(cBuf, nSamples, &cFiltDecim3, 3);
+	// Convert format from complex double to stereo pairs of 16-bit PCM samples.
+	// Buffer samples until UDP_SEND_INT16 are available, and then send the block.
+	for (i = 0; i < nSamples; i++) {
+		buffer[buffer_index++] = (int16_t)(creal(cBuf[i]) * (double)CLIP16 / CLIP32);
+		buffer[buffer_index++] = (int16_t)(cimag(cBuf[i]) * (double)CLIP16 / CLIP32);
+		if (buffer_index >= UDP_SEND_INT16) {
 			sent = send(control_head_sound_socket, (const char *)buffer, buffer_index * 2, 0);
 			if (sent != buffer_index * 2)
 				QuiskPrintf("send_remote_mic_sound_socket(), send(): %s\n", strerror(errno));
@@ -393,13 +217,17 @@ void send_remote_mic_sound_socket(complex double * cSamples, int nSamples)
 // Send radio speaker sound from the remote radio to the control head
 void send_remote_radio_sound_socket(complex double * cSamples, int nSamples)
 {	// Send nSamples samples.  Each sample is sent as two shorts (4 bytes) of L/R audio data.
-	int i, sent;
-	short sound_lr[MAX_SAMPLES_FOR_REMOTE_SOUND * 2]; // Limit Tx to 60,000 bytes, 30,000 shorts, 15,000 samples
-	int udp_size = 0;	// Keep track of UDP payload size, in shorts
+	int i, N, sent;
+	static int16_t sound_lr[UDP_SEND_INT16];
+	static int udp_size = 0;	// Keep track of UDP payload size, in shorts
 	char buf[RX_BUFFER_SIZE];	// For startup message
 	int recv_len;
 	int retval;
 	SOCKET * sock = &remote_radio_sound_socket;
+	static struct quisk_cHB45Filter HalfBand;
+	static struct quisk_cFilter cFiltDecim3;
+	static int init_filters=1, size_cBuf=0;
+	static complex double * cBuf=NULL;
 
 #if REMOTE_DEBUG > 0 //BMC debug
 	// measure/monitor tools:
@@ -419,6 +247,16 @@ void send_remote_radio_sound_socket(complex double * cSamples, int nSamples)
 	if (*sock == INVALID_SOCKET)
 		return;
 
+	if (init_filters) {
+		init_filters = 0;
+		memset(&HalfBand, 0, sizeof(struct quisk_cHB45Filter));
+		quisk_filt_cInit(&cFiltDecim3, quiskAudio24p3Coefs, sizeof(quiskAudio24p3Coefs)/sizeof(double));
+	}
+	if (nSamples > size_cBuf) {
+		size_cBuf = nSamples;
+		cBuf = (complex double *)realloc(cBuf, size_cBuf * sizeof(complex double));
+	}
+
 #if REMOTE_DEBUG > 0 //BMC debug
 	callcount++;
 #endif
@@ -461,39 +299,40 @@ void send_remote_radio_sound_socket(complex double * cSamples, int nSamples)
 		}
 	}
 
-	if (nSamples > MAX_SAMPLES_FOR_REMOTE_SOUND) {
-		QuiskPrintf("send_remote_sound_socket():  nSamples %i > MAX_SAMPLES_FOR_REMOTE_SOUND 15,000, trimming to MAX\n", nSamples);
-		nSamples = MAX_SAMPLES_FOR_REMOTE_SOUND;
-	}
-
+	memcpy(cBuf, cSamples, nSamples * sizeof(complex double));	// Do not alter cSamples
+	// Reduce sample rate from 48 to 8 ksps
+	nSamples = quisk_cDecim2HB45(cBuf, nSamples, &HalfBand);
+	nSamples = quisk_cDecimate(cBuf, nSamples, &cFiltDecim3, 3);
 	// Convert format from complex double to stereo pairs of 16-bit PCM samples, send to client
 	for (i = 0; i < nSamples; i++) {
-		sound_lr[udp_size++] = (short)(creal(cSamples[i]) * (double)CLIP16 / CLIP32);
-		sound_lr[udp_size++] = (short)(cimag(cSamples[i]) * (double)CLIP16 / CLIP32);
-	}
-	if (udp_size > 0) {
-		sent = send(*sock, (char *)sound_lr, udp_size * 2, 0);
-		if (sent != udp_size * 2)
-			QuiskPrintf("send_remote_sound_socket(), send(): %s\n", strerror(errno));
+		sound_lr[udp_size++] = (int16_t)(creal(cBuf[i]) * (double)CLIP16 / CLIP32);
+		sound_lr[udp_size++] = (int16_t)(cimag(cBuf[i]) * (double)CLIP16 / CLIP32);
+		if (udp_size >= UDP_SEND_INT16) {
+			N = UDP_SEND_INT16;
+			udp_size = 0;
+			sent = send(*sock, (char *)sound_lr, N * 2, 0);
+			if (sent != N * 2)
+				QuiskPrintf("send_remote_sound_socket(), send(): %s\n", strerror(errno));
 #if REMOTE_DEBUG > 0 //BMC debug
-		else {
-			sampcount += sent/4;
-			packets_sent++;
+			else {
+				sampcount += sent/4;
+				packets_sent++;
 #if REMOTE_DEBUG > 1 //BMC debug
-			now = QuiskTimeSec();
-			QuiskPrintf("%f, send_remote_sound_socket(): now - prior = %f, samples = %u, sampcount = %lu\n",
-				now, now - prior_packet_ts, sent / 4, sampcount);
-			prior_packet_ts = now;
+				now = QuiskTimeSec();
+				QuiskPrintf("%f, send_remote_sound_socket(): now - prior = %f, samples = %u, sampcount = %lu\n",
+					now, now - prior_packet_ts, sent / 4, sampcount);
+				prior_packet_ts = now;
 #endif
+			}
+#endif
+		}
+#if REMOTE_DEBUG > 1 //BMC debug
+		else {
+			now = QuiskTimeSec();
+			QuiskPrintf("%f, send_remote_sound_socket(): nSamples = %i\n", now, nSamples);
 		}
 #endif
 	}
-#if REMOTE_DEBUG > 1 //BMC debug
-	else {
-		now = QuiskTimeSec();
-		QuiskPrintf("%f, send_remote_sound_socket(): nSamples = %i\n", now, nSamples);
-	}
-#endif
 #if REMOTE_DEBUG > 0 //BMC debug
 	if (callcount >= 200) {
 		double new_ts = QuiskTimeSec();
@@ -570,6 +409,10 @@ void send_graph_data(double * fft_avg, int fft_size, double zoom, double deltaf,
 			free(pixels);
 		pixels = (double *)malloc(n_pixels * sizeof(double));
 	}
+	if ( ! fft_avg) {	// send dummy graph data
+		send(remote_radio_graph_socket, "dum", 3, 0);
+		return;
+	}
 	copy2pixels(pixels, control_head_data_width, fft_avg, fft_size, zoom, deltaf, fft_sample_rate);
 	// Send multiple 16-bit data blocks: {flags, sequence}, block number, 16-bit graph data
 	// 8-bit flags:
