@@ -73,7 +73,7 @@ class Remot:	# Remote comtrol base class
     self.graph_data_port = self.remote_ctl_base_port + 1
     self.remote_radio_sound_port = self.remote_ctl_base_port + 2
 
-    self.cw_delay = 0.020	# time delay to absorb WiFi jitter, in secs
+    self.cw_delay_secs = 0.020	# time delay to absorb WiFi jitter, in secs
     self.cw_phrase_begin_ts = None	# timestamp of beginning of cw phrase
     self.cw_next_event_ts = None
     self.cw_next_keydown = None
@@ -244,8 +244,12 @@ class Remot:	# Remote comtrol base class
       if command == 'HEARTBEAT':
         self.remote_ctl_heartbeat_ts = time.time()
         continue
-      # Ignore the On/Off button, Help buttons
-      if command in ("On", ".."):
+      # Ignore the On/Off button, Help buttons, Small window pop buttons
+      if command in ("On", "..", "bandBtnGroup", "screenBtnGroup", "modeButns", "Scope", "Config", "RX Filter", "Help"):
+        continue
+      if DEBUG: print("Remote receive:", cmd_text)
+      # Look for radio buttons
+      if self.ProcessRadioBtn(command, self.cmd_text):
         continue
       # buttons in idName2Button
       btn = self.app.idName2Button.get(args[0], None)
@@ -278,7 +282,6 @@ class Remot:	# Remote comtrol base class
         if source == "NewDecim":
           self.app.config_screen.config.btn_decimation.SetSelection(var_decim_index)
           sample_rate = self.VarDecimSet(var_decim_index)
-          print ("sample_rate", sample_rate)
           self.app.OnBtnDecimation(rate=sample_rate)
       # Json function call
       elif command == 'JsonAppFunc':
@@ -323,7 +326,7 @@ class Remot:	# Remote comtrol base class
           else:
             # Begin new cw phrase; any prior phrase should be done by now.
             # Set up first cw event to be ready to execute.
-            self.cw_begin_phrase_ts = ts + self.cw_delay
+            self.cw_begin_phrase_ts = ts + self.cw_delay_secs
         cw_new_event_ts = self.cw_begin_phrase_ts + cw_event_ts 
         if not self.cw_next_event_ts:
           self.cw_next_event_ts = cw_new_event_ts
@@ -379,3 +382,58 @@ class Remot:	# Remote comtrol base class
     self.cw_next_keydown = None
     self.cw_key_down = 0
     QS.set_remote_cwkey(0)
+
+  def ProcessRadioBtn(self, command, cmd_text):
+    # Large and Small format screens send different button events for radio buttons.
+    # Band buttons:
+    if command in self.conf.BandList:
+      self.app.bandBtnGroup.SetLabel(command, True)
+      return True		# We processed this command
+    if command in ("Audio", "Time"):
+      self.app.bandBtnGroup.SetLabel(command, True)
+      return True		# We processed this command
+    Mode = self.app.modeButns.SetLabel
+    Screen = self.app.screenBtnGroup.SetLabel
+    # Mode buttons: process both formats:
+    if cmd_text in ("CW U/L;0", "CWL;0", "CWL;1"):
+      Mode("CWL", True)
+    elif cmd_text in ("CW U/L;1", "CWU;0", "CWU;1"):
+      Mode("CWU", True)
+    elif cmd_text in ("SSB U/L;0", "LSB;0", "LSB;1"):
+      Mode("LSB", True)
+    elif cmd_text in ("SSB U/L;1", "USB;0", "USB;1"):
+      Mode("USB", True)
+    elif cmd_text in ("AM;0", "AM;1"):
+      Mode("AM", True)
+    elif cmd_text in ("FM;0", "FM;1"):
+      Mode("FM", True)
+    elif cmd_text in ("DGT;0", "DGT-U;0", "DGT-U;1"):
+      Mode("DGT-U", True)
+    elif cmd_text in ("DGT;1", "DGT-L;0", "DGT-L;1"):
+      Mode("DGT-L", True)
+    elif cmd_text in ("DGT;2", "DGT-FM;0", "DGT-FM;1"):
+      Mode("DGT-FM", True)
+    elif cmd_text in ("DGT;3", "DGT-IQ;0", "DGT-IQ;1"):
+      Mode("DGT-IQ", True)
+    elif cmd_text in ("FDV;0", "FDV-U;0", "FDV-U;1"):
+      Mode("FDV-U", True)
+    elif cmd_text in ("FDV;1", "FDV-L;0", "FDV-L;1"):
+      Mode("FDV-L", True)
+    # Screen buttons: process both formats:
+    # Due to ambiguous received commands, the setting is always "Graph" or "WFall" without "P1 or "P2".
+    # The P1 and P2 are handled at the control head so this shouldn't matter.
+    elif cmd_text in ("Graph;0", "Graph;0", "Graph;1"):
+      Screen("Graph", True)
+    elif cmd_text in ("Graph;1", "GraphP1;0", "GraphP1;1"):
+      Screen("Graph", True)
+    elif cmd_text in ("Graph;2", "GraphP2;0", "GraphP2;1"):
+      Screen("Graph", True)
+    elif cmd_text in ("WFall;0", "WFall;0", "WFall;1"):
+      Screen("WFall", True)
+    elif cmd_text in ("WFall;1", "WFallP1;0", "WFallP1;1"):
+      Screen("WFall", True)
+    elif cmd_text in ("WFall;2", "WFallP2;0", "WFallP2;1"):
+      Screen("WFall", True)
+    else:
+      return False
+    return True		# We processed this command
