@@ -7,25 +7,28 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
 
-import math, wx
+import math, wx, time
+from configure import ComboCtrl
+
 
 class BottomWidgets:	# Add extra widgets to the bottom of the screen
+  REG_ANTENNA_TUNER = 7
   def __init__(self, app, hardware, conf, frame, gbs, vertBox):
     self.config = conf
     self.hardware = hardware
     self.application = app
     self.start_row = app.widget_row			# The first available row
     self.start_col = app.button_start_col	# The start of the button columns
+    self.tuner_time = 0
     self.Widgets_0x06(app, hardware, conf, frame, gbs, vertBox)
   def Widgets_0x06(self, app, hardware, conf, frame, gbs, vertBox):
     self.num_rows_added = 1
     start_row = self.start_row
-    b = app.QuiskCheckbutton(frame, self.OnAGC, 'RfAgc')
-    if hardware.hermes_code_version >= 40:
-      b.Enable(False)
-    gbs.Add(b, (start_row, self.start_col), (1, 2), flag=wx.EXPAND)
-    bw, bh = b.GetMinSize()
-    init = self.config.hermes_LNA_dB
+    self.atu_ctrl = ComboCtrl(frame, "ATU", ["Tune", "Bypass"], True)
+    gbs.Add(self.atu_ctrl, (start_row, self.start_col), (1, 2), flag=wx.EXPAND)
+    bw, bh = self.atu_ctrl.GetMinSize()
+    frame.Bind(wx.EVT_COMBOBOX_CLOSEUP, self.OnAtu)
+    init = app.hermes_LNA_dB
     self.sliderLNA = app.SliderBoxHH(frame, 'RfLna %d dB', init, -12, 48, self.OnLNA, True)
     self.sliderLNA.idName = "RfLna"
     app.midiControls["RfLna"]	= (self.sliderLNA,	self.OnLNA)
@@ -71,13 +74,25 @@ class BottomWidgets:	# Add extra widgets to the bottom of the screen
       szr.Add(self.text_pa_current, 0, flag=flag)
       szr.Add(self.text_fwd_power, 0, flag=flag)
       szr.Add(self.text_swr, 0, flag=flag)
-  def OnAGC(self, event):
-    btn = event.GetEventObject()
-    value = btn.GetValue()
-    self.hardware.ChangeAGC(value)
+  def OnAtu(self, event):
+    if not self.hardware.io_board.have_IO_Board:
+      self.atu_ctrl.SetText("No ATU")
+      self.tuner_time = 0
+    elif self.atu_ctrl.GetValue() == "Tune":
+      self.atu_ctrl.SetText("Tuning")
+      self.tuner_time = time.time()
+      self.hardware.WriteI2C(0x7d, 0x1D, self.REG_ANTENNA_TUNER, 1)
+    else:
+      self.atu_ctrl.SetText("Bypass")
+      self.tuner_time = 0
+      self.hardware.WriteI2C(0x7d, 0x1D, self.REG_ANTENNA_TUNER, 2)
+      if self.application.spotButton.GetValue():
+        self.application.spotButton.SetValue(False, True)
+    self.sliderLNA.SetFocus()
   def OnLNA(self, event=None):
     value = self.sliderLNA.GetValue()
     self.hardware.ChangeLNA(value)
+    self.application.hermes_LNA_dB = value
   def Code2Temp(self):		# Convert the HermesLite temperature code to the temperature
     temp = self.hardware.hermes_temperature
     # For best accuracy, 3.26 should be a user's measured 3.3V supply voltage.
@@ -134,6 +149,31 @@ class BottomWidgets:	# Add extra widgets to the bottom of the screen
     else:
       text = " SWR  ---"
     self.text_swr.SetLabel(text)
+    if self.tuner_time > 0:
+      ret = self.hardware.io_board.Receive(self.REG_ANTENNA_TUNER)
+      if ret:
+        code = ret[0]
+        if code == 0:
+          t = "ATU OK"
+          self.tuner_time = 0
+        elif code == 0xEE:
+          t = "ATU RF"
+        elif code >= 0xF0:
+          t = "Err 0x%X" % code
+          self.tuner_time = 0
+        else:
+          t = "ATU 0x%X" % code
+        self.atu_ctrl.SetText(t)
+        if code == 0xEE:
+          if not self.application.spotButton.GetValue():
+            self.application.spotButton.SetValue(True, True)
+        elif self.application.spotButton.GetValue():
+          self.application.spotButton.SetValue(False, True)
+      else:
+        code = -1
+      if code != 0 and self.tuner_time > 0 and time.time() - self.tuner_time > 20:
+        self.tuner_time = 0
+        self.atu_ctrl.SetText("ATU Err")
   def OnTextDataMenu(self, event):
     btn = event.GetEventObject()
     btn.PopupMenu(self.text_data_menu, (0,0))
